#ifndef __IKONKE_MODULE_UART_H_______________________________
#define __IKONKE_MODULE_UART_H_______________________________

#include "../general/ikk-module-def.h"
#include "drv_uart.h"

// MSG BUFFER OF NODE MAXLENGTH
#define UMSG_NODE_SIZE					96
// Maximum number of message nodes in the queue
#define UMSG_NODE_MAX					4
// Maximum Cache Length of Message Receiver
#define UART_RECEIVER_CACHE_SIZE		128

// 
typedef enum { EQU_RECV = 0, EQU_SENT }QueueEm;

// uart recv/send buffer node
typedef struct tag_uart_message_node {
	unsigned char buffer[UMSG_NODE_SIZE];
	uint16_t length;
	// just for sending, Used to match instruction responses. For example, according to the instruction opcode!
	unsigned char matcher[4];
	unsigned char matcher_offset;
	unsigned char matcher_n;
	// just for sending, need to dispatch the response message?
	bool bRspNeedtoDispatch;
	// Attempt to send number of times, decreases to 0 when invalid!!!
	unsigned char sent_try;
	unsigned char *ack;
	unsigned char ack_length;
}UMsgNodeSt;

typedef struct tag_uart_message_queue {
	UMsgNodeSt node[UMSG_NODE_MAX];
	unsigned char counter;
}UMsgQueueSt;

typedef bool (*PFUNC_CRC)(uint8_t *pdat, int length );
typedef void (*PFUNC_INCOMING_CALLBACK)(UMsgNodeSt *pMsgNode );

typedef struct tag_uart_manager_st {
	GPIO_PinTypeDef Rxpin;
	GPIO_PinTypeDef Txpin;

	uint8_t recv_header[4];
	uint8_t recv_header_n;
	// head detection assistant chunk!
	uint8_t header_chunk[4];
	uint8_t header_chunk_index;

	// length occupy index,0xff is valided!
	uint8_t buffer_len_index;
	uint8_t additional_len;

	PFUNC_CRC pfunc_crc;
	PFUNC_INCOMING_CALLBACK pfunc_incoming;
	// support for sleepy device. when waking up, the serial port receives automatically,
	// and after receiving, it sleeps actively.
	bool bSleepSupport;
	// support for sleepy device. the countdown ends and goes to sleep!
	uint16_t u16SleepCounter;
}UartManagerSt;

kk_err_t kUartMsgSent(UMsgNodeSt *pMsgNode );
/*	DESP: serial usart driver init interface.
 *	PARAM[recv_header]: Recognition Sequence of recv effective data frame header and detection of Data Frame start.
 *	PARAM[recv_header_n]: the length of recv data frame header.
 *	PARAM[buffer_len_index]: the length of paload.
 *	PARAM[addtional_len]: the length of added
 *	PARAM[pfunc_crc]: CRC function.
 *	PARAM[pfunc_incoming]: Callback function when valid data frame is detected, provided by youself.
 *	PARAM[bSleepSupport]: support for sleepy device function.
 * 	Auth: dingmz_frc.20190620.
 * */
kk_err_t kUartModuleInit(GPIO_PinTypeDef Txpin,GPIO_PinTypeDef Rxpin, uint32_t rate, UART_ParityTypeDef parity, UART_StopBitTypeDef stopBits
		, uint8_t recv_header[], uint8_t recv_header_n,uint8_t buffer_len_index,uint8_t addtional_len
		, PFUNC_CRC pfunc_crc, PFUNC_INCOMING_CALLBACK pfunc_incoming
		, bool bSleepSupport );

#if 0
void UART_TEST(void);
#endif


#endif

